/**
 * Movie Controller
 * @author Belkacem Alidra <belkacem.alidranetoxygen.ch>
 */
TicketackControllers.controller('MovieCtrl', function(
    $scope, $rootScope, $state, $stateParams,
    $ionicAnalytics, $ionicModal, $sce, $cordovaLaunchNavigator,
    $ionicScrollDelegate, $ionicSlideBoxDelegate,
    Config, Movies, Cart, Tickets, LocalStorage, Toast) {

    /* The current movie */
    $scope.movie;
    /* Slide box active slide */
    $scope.active_slide                = 0;
    /* The screening selected to add to the cart*/
    $scope.screening_selected_for_cart;

    /* Pricings to add to cart as an object with the pricings
     * as keys and the quantities as values
     */
    $scope.pricings_to_add_to_cart;

    /* Get the current pass, if any */
    $scope.current_pass = Tickets.current_pass;

    /* The movie is loaded:
     *  - on beforeEnter, from the Movies cache, if the cache
     *    exists (see: Movies.loaded). This is the Movies.get result.
     *  - on afterEnter, if the cache didn't exist, a call to the API is made, so the
     *    movie is updated in the callback function.
     */
    $scope.$on('$ionicView.beforeEnter', function(e) {
        $scope.movie = Movies.get($stateParams.movie_id, /* from_cache */true, function(err, movie) {
            if (err)
                Toast.error();

            $scope.movie = update_screenings_infos(movie);
            $ionicSlideBoxDelegate.update();
        });
    });
    $scope.$on('$ionicView.afterEnter', function(e) {
        Movies.get($stateParams.movie_id,  /* from_cache */false, function(err, movie) {
            if (err)
                Toast.error();

            $scope.movie = update_screenings_infos(movie);
            $ionicSlideBoxDelegate.update();
        });
    });

    /**
     * For each of this movie screenings, we check if:
     *  - it is running, finished or in future
     *  - it has available seats
     *  - it is in the favorites
     *  - it is already booked
     *  - it has a linked screening
     *
     * We do it each time we see this movie.
     */
    var update_screenings_infos = function(movie, callback) {
        _.each(movie.screenings, function(s) {

            /* Check if running or finished */
            if (moment().isAfter(s.start_at)) {
                if (moment().isAfter(s.stop_at)) {
                    s.is_finished = true;
                }
                else
                    s.is_running  = true;
            }

            /* Check available seats */
            if (s.seats.confirmed >= s.seats.total)
                s.is_complete = true;

            /* Check if i is in favorites */
            _.each(LocalStorage.getArray('favorites'), function(f) {
                if (f.screening_id == s._id)
                    s.is_in_favs = true;
            })

            /* Check if it is already booked */
            s.already_booked = 0;
            $scope.current_pass = Tickets.get_current_pass();
            if ($scope.current_pass &&
                !_.isEmpty($scope.current_pass.bookings)) {
                _.each($scope.current_pass.bookings, function(b) {
                    if (b.screening_id == s._id)
                        s.already_booked ++;
                })
            }

            /**
             * Check if it has a linked screening:
             * In this case, the user can't book this screening but
             * only the "linked" one.
             */
            if (s.opaque.booking_mode == "linked") {
                Movies.screening(null, s.opaque.note, function(err, om, os) {
                    if (err)
                        return false;
                    /* If the print note contains the same screening id */
                    if (om._id == movie._id)
                        s.opaque.booking_mode = "";
                    else
                        s.linked_screening_movie_id = om._id;
                });
            }
        });

        return movie;
    };

    /* Called on each slide change */
    $scope.slideHasChanged = function(index) {
        $scope.active_slide = index;
        $ionicScrollDelegate.scrollTop();
    };
    /* Change slide, on button click */
    $scope.change_slide = function(index) {
        $scope.active_slide = index;
        $ionicScrollDelegate.scrollTop();
    };

    /* Show a screening pricings */
    $scope.show_screening_pricings = function(screening) {
        $scope.screening_selected_for_cart = screening;

        $scope.pricings_to_add_to_cart = {};
        _.each($scope.screening_selected_for_cart.pricings, function(pricing, key) {
            $scope.pricings_to_add_to_cart[key] = 0;
        });
    };
    /* Hide the screening pricings */
    $scope.hide_screening_pricings = function() {
        $scope.screening_selected_for_cart = undefined;
        $scope.pricings_to_add_to_cart     = undefined;
    };

    /* Increase the quantity for a pricing to add to cart */
    $scope.add_one_for_pricing = function(key) {
        $scope.pricings_to_add_to_cart[key] ++;
    };
    /* Decrease the quantity for a pricing to add to cart */
    $scope.remove_one_for_pricing = function(key) {
        $scope.pricings_to_add_to_cart[key] = Math.max(
            0,
            $scope.pricings_to_add_to_cart[key] - 1
        );
    };

    /* Add the selected prigings to cart */
    $scope.add_to_cart = function(callback) {
        /* Let's track this event in ionic analytics */
        $ionicAnalytics.track('Add to cart', {
            item_id: $scope.screening_selected_for_cart._id,
            item_name: $scope.screening_selected_for_cart.title.original
        });

        Toast.loading();

        Cart.add(
            $scope.screening_selected_for_cart._id,
            $scope.pricings_to_add_to_cart,
            function(err) {
                Toast.hide();
                if (err)
                    Toast.error(err.error.data);
                else
                    Toast.success();

                $scope.hide_screening_pricings();
                Cart.get();
            });
    };

    /* Book a screening on the user ticket
     * We first check the bookability and if ok,
     * we book the screening.
    */
    $scope.book_on_ticket = function(screening) {
        Toast.loading();
        /* Let's check the screening bookability */
        Tickets.check_bookability(screening._id, function(err, data) {
            Toast.hide();
            if (err)
                Toast.error('Une erreur est survenue');
            else {
                if (!data.screening_exists)
                    Toast.error('Une erreur est survenue');
                else if (!data.ticket_logged_in)
                    Toast.error('Veuillez vous connecter avec votre abonnement');
                else if (!data.screening_seats)
                    Toast.error('Il n\'y a plus de places pour cet événement');
                else if (data.screening_already_booked == true && !data.ticket_can_book_screening)
                    return $scope.cancel_booking(screening);
                else if (!data.screening_already_booked == true && !data.ticket_can_book_screening)
                    Toast.error('Vous ne pouvez pas réserver pour cet événement');
                // Everything OK
                else {
                    /* Let's book the screening */
                    Tickets.book_on_ticket(screening._id, function(err, data) {
                        if (err)
                            return Toast.error('Une erreur est survenue');

                        /* Store in the screening the info that it's booked */
                        screening.already_booked ++;

                        Toast.success(data);

                        /* Reload ticket after booking */
                        Tickets.login(Tickets.get_current_pass(),
                            function(err, pass) {
                                if (!err)
                                    $scope.current_pass = pass;
                            }
                        );
                    });
                }
            }
        });
    };

    /* Cancel a booking */
    $scope.cancel_booking = function(screening) {
        /* let's find the booking corresponding to this screening */
        _.each($scope.current_pass.bookings, function(booking) {
            if (booking.screening_id == screening._id) {
                var booking_id = booking._id;

                Tickets.cancel_booking(booking_id, function(err, data) {
                    if (err)
                        return Toast.error('Une erreur est survenue');

                    /* Store in the screening the info that it's no more booked */
                    screening.already_booked --;
                    Toast.success();
                })
            }
        });
    };

    /* Add a screening to the favorites, stored in the local storage */
    $scope.add_to_favs = function(screening) {
        /* Let's track this event */
        $ionicAnalytics.track('Add to favorites', {
            item_id: screening._id,
            item_name: screening.title.original
        });

        var favs = LocalStorage.getArray('favorites');
        favs.push({
            "movie_id": $scope.movie._id,
            "screening_id": screening._id,
            "start_at": screening.start_at
        })
        LocalStorage.setArray('favorites', favs);
        screening.is_in_favs = true;
    }

    /* Remove a screening from the favorites */
    $scope.remove_from_favs = function(screening) {
        var new_favs = [],
        favs     = LocalStorage.getArray('favorites');
        _.each(favs, function(f, index) {
            if (f.screening_id != screening._id)
            new_favs.push(f);
        })
        LocalStorage.setArray('favorites', new_favs);
        screening.is_in_favs = false;
    };

    if (!ionic.Platform.is('android')) {
        $scope.close_map = function() {
            $scope.modal.hide();
        };
        //Cleanup the modal when we're done with it!
        $scope.$on('$destroy', function() {
            $scope.modal.remove();
        });
        // Execute action on hide modal
        $scope.$on('modal.hidden', function() {
            $scope.location_name = null;
            $scope.map_url       = null;
        });

        // Execute action on remove modal
        $scope.$on('modal.removed', function() {
            $scope.location_name = null;
            $scope.map_url       = null;
        });

        $ionicModal.fromTemplateUrl('templates/map.html', {
            scope: $scope,
            animation: 'slide-in-up'
        }).then(function(modal) {
            $scope.modal = modal;
        });
    }

    /* Open a modal with Google Maps on ios
     * and open Google Maps app on Android
     */
    $scope.navigate_to = function(screening) {

        var coords = screening.cinema_hall.coordinates;
        if (_.isEmpty(coords))
            return false;

        var lat  = coords.lat;
        var long = coords.long;

        if (ionic.Platform.is('android')) {
            $cordovaLaunchNavigator.navigate(
                [ coords.lat, coords.long],
                null,
                function() {},
                function(error) {}
            );
        }
        else {
            var key  = Config.google_maps_api.ios_key;

            $scope.location_name = screening.cinema_hall.name;
            $scope.map_url = $sce.trustAsResourceUrl(
                "https://www.google.com/maps/embed/v1/place?key=" + key +
                "&q=" + lat + "," + long
            );

            $scope.modal.show();
        }
      };
  });
