/**
 * Cart Service
 *
 * Uses the Tiketack API following calls:
 *      - get the cart
 *          GET  /cart/view_json
 *      - add to the cart
 *          POST /screening/buy/:{screening_id}
 *      - remove from the cart
 *          GET  /cart/remove/:index
 *
 * To bypass the Captcha mechanism, a token is sent by the server when we get
 * the cart. This token is sent in all following calls as header X-Cap-Token.
 * To maintain the session, the PHPSESSID is also sent by the server when we
 * get the cart.
 *
 * @author Belkacem Alidra <belkacem.alidra@netoxygen.ch>
 */
TicketackServices.factory(
    'Cart',
    [ '$rootScope', '$http', '$cordovaNetwork', 'Config', 'Movies',
    function($rootScope, $http, $cordovaNetwork, Config, Movies) {

        return {
            /* Captcha bypass token */
            token: null,
            /* PHPSESSID */
            session_id: null,

            /**
             * Load the user cart
             *
             * We receive from the server:
             *   - the cart object
             *   - the Captcha bypass token
             *   - the PHPSESSID
             *
             * @param function: Callback
             */
            get: function(callback) {
                var url = "/cart/view_json";

                /* Append PHPSESSID if exists to keep the session */
                if ( _.isString(this.session_id) &&
                    !_.isEmpty(this.session_id) &&
                    "null" != this.session_id
                )
                    url += '?PHPSESSID=' + this.session_id;

                var that = this;

                return this.http_get(url, function(err, cart) {
                    if (!err) {
                        cart.view_cart_url     += "?PHPSESSID=" + that.session_id +
                                                  "&from_app=1";
                        cart.validate_cart_url += "?PHPSESSID=" + that.session_id +
                                                  "&from_app=1";
                        cart.buy_pass_url       = Config.ticketack_url +
                                                  "/pass/new" +
                                                  "?PHPSESSID=" + that.session_id +
                                                  "&from_app=1";;

                        cart.view_ticket_url    = function(ticket_id) {
                            return  Config.ticketack_url +
                                    "/ticket/pdf/uuid/" + ticket_id;
                        };

                        that.token	    = cart.token;
                        that.session_id = cart.session_id;

                        $rootScope.cart             =  cart;
                        $rootScope.nb_items_in_cart =  cart.items.length;

                        /* Add the movie and screening in each cart item */
                        if (!_.isEmpty(cart.items)) {
                            _.each(cart.items, function(item) {
                                if ("screening" == item.type) {
                                    Movies.screening(null, item.screening_id, function(err, movie, screening) {
                                        if (!err) {
                                            item.movie     = movie;
                                            item.screening = screening;
                                        }
                                    });
                                }
                                else if ("pass" == item.type) {
                                    item.poster = Config.default_poster_url;
                                }
                            });
                        }

                        return (callback && callback(null, cart));
                    }

                    return (callback && callback(err));
                });
            },

            /**
             * Add a screening to cart
             *
             * @param mixed: The screening id
             * @param: object: The pricings object
             * @param function: Callback
             */
            add: function(id, pricing, callback) {

                return this.http_post(
                    '/screening/buy/' + id,
                    { "pricing": pricing },
                    function(err, infos) {
                        if (!err)
                            return (callback && callback(null, infos.data));

                        return (callback && callback(err));
                    }
                );
            },

            /**
             * Remove a screening from cart
             *
             * @param number: The cart item id
             * @param function: Callback
             */
            remove: function(index, callback) {
                var url = '/cart/remove?index=' + index +
                          '&PHPSESSID=' + this.session_id +
                          '&pos_client_side_request=true';

                return this.http_get(url, callback);
            },

            /**
             * Make an http GET to the eshop.
             * Ensure that the Captcha bypass token and PHPSESSID
             * are always sent.
             *
             * @param string: The url suffix: only the ressource part
             * @param function: Callback
             */
            http_get: function(url, callback) {
                if ($rootScope.is_offline)
                    return callback({"error": "NO_CONNECTION"});

                $http({
                    "method": 'get',
                    "url": Config.ticketack_url + url,
                    "format": 'json',
                    "headers": {
                        'X-Cap-Token': this.token
                    },
                    "withCredentials": true,
                    "timeout": 6000
                }).then(function successCallback(infos) {
                    callback && callback(null, infos.data);
                }, function errorCallback(response) {
                    callback && callback({"error": response});
                });
            },

            /**
             * Make an http GET to the eshop.
             * Ensure that the Captcha bypass token and PHPSESSID
             * are always sent.
             *
             * @param string: The url suffix: only the ressource part
             * @param object: the data to send
             * @param function: Callback
             */
            http_post: function(url, data, callback) {
                if ($rootScope.is_offline)
                    return callback({"error": "NO_CONNECTION"});
                $http({
                    "method": 'post',
                    "url": Config.ticketack_url + url,
                    "format": 'json',
                    "data": angular.extend({
                        "pos_client_side_request": true,
                        "PHPSESSID": this.session_id
                    }, data),
                    "headers": {
                        'X-Cap-Token': this.token
                    },
                    "withCredentials": true,
                    "timeout": 6000
                }).then(function successCallback(infos) {
                    callback && callback(null, infos);
                }, function errorCallback(response) {
                    callback && callback({"error": response});
                });
            }
        }
    }
]);
