/**
 * Movies Service
 *
 * Provides access to Ticketack screenings and movies.
 * The movies are built using the screenings "films", so the only call used is:
 * GET /api/screenings
 *
 * @author Belkacem Alidra <belkacem.alidra@netoxygen.ch>
 */
TicketackServices.factory(
    'Movies',
    function(Config, _, $rootScope, $resource, $sce, $cordovaNetwork) {

    /**
     * Ressource used to communicate with the engine.
     * It ensures to always send the configured api key.
     */
    var Screenings = $resource(Config.ticketack_url + '/api/screenings', /* default params */null, {
        "all": {
            "method": "GET",
            "headers": {
                "X-API-Key": Config.api_key
            },
            "isArray": true,
            "timeout": 6000
        }
    });

    /**
     * Load the movies and screenings from Ticketack and enhance them:
     *  - creates each movie shortline, used in the movie screen.
     *  - check the movies posters, trailers and section.
     *  - check if the screenings are bookable depending on their booking mode.
     *  - filters the screenings pricings. Allow only the POS role.
     *
     * @param function: Callback which receives the movies and days
     */
    var load_movies = function(callback) {
        var self   = this,
        movies = {},
        days   = [];

        if ($rootScope.is_offline) {
            return callback({"error": "NO_CONNECTION"}, movies, days);
        }

        var screenings = Screenings.all(function() {
            var i = 0;
            _.each(screenings, function(s) {

                /* Fill the days array based on the screenings */
                var day = moment(s.start_at).format('YYYY-MM-DD');
                if (!_.contains(days, day))
                    days.push(day);

                /**
                 * Check the screening booking mode
                 *  - ""       => Normal booking
                 *  - "free"   => Free entrance
                 *  - "email"  => Booking by email
                 *  - "no"     => No booking
                 *  - "pos"    => Booking only on POS
                 *  - "linked" => Booking is only possible on another screening
                 *  - "ignore" => Ignore screening
                 */
                if  (s.opaque.booking_mode == 'ignore') {
                    return false;
                }
                s.bookings_enabled = _.isEmpty(s.opaque.booking_mode);
                s.in_favs_enabled  = s.bookings_enabled ||
                                    _.contains(["free", "email", "pos"],  s.opaque.booking_mode);

                /* Build the movies from the screenings */
                _.each(s.films, function(f) {
                    if (_.isUndefined(movies[f._id])) {
                        f.screenings = [];

                        /* Pre-build movie short line */
                        f.short_line = [];
                        if (!_.isEmpty(f.opaque.directors)) {
                            var r = f.opaque.directors[0];
                            f.short_line.push(r.surname + " " + r.name);
                        }
                        if (!_.isEmpty(f.opaque.countries)) {
                            var countries = [];
                            _.each(f.opaque.countries, function(c) {
                                countries.push(c.name);
                            });
                            countries.length && f.short_line.push(countries.join('/'));
                        }
                        if (!_.isEmpty(f.opaque.cinech) &&
                            !_.isEmpty(f.opaque.cinech.release_date))
                            f.short_line.push(f.opaque.cinech.release_date.substr(0, 4));

                        if (!_.isEmpty(f.opaque.duration))
                            f.short_line.push(f.opaque.duration + "'");

                        f.short_line = f.short_line.join(", ");

                        /* Check movie poster */
                        if (!_.isEmpty(f.opaque.posters)) {
                            f.has_poster = true;
                            f.poster     = f.opaque.posters[0];
                        }
                        if (_.isEmpty(f.poster)) {
                            var default_poster = Config.default_poster_url;
                            if (!_.isEmpty(default_poster)) {
                                f.has_poster = true;
                                f.poster     = default_poster;
                            }
                        }

                        /* Check movie trailer */
                        if (!_.isEmpty(f.opaque.trailers) &&
                            _.isString(f.opaque.trailers[0].url) &&
                            !_.isEmpty(f.opaque.trailers[0].url)) {
                                f.has_trailer = true;
                                f.trailer     = $sce.trustAsResourceUrl(f.opaque.trailers[0].url);
                        }

                        /* Movie extra infos */
                        f.extra_infos = {};
                        _.each(['directors', 'producers', 'actors', 'writers'], function(key) {
                            if (!_.isEmpty(f.opaque[key])) {
                                var tmp = [];
                                _.each(f.opaque[key], function (person) {
                                    tmp.push(person.name + " " + person.surname);
                                })
                                f.extra_infos[key] = tmp.join(', ');
                            }
                        });
                        if (!_.isEmpty(f.opaque.crew)) {
                            var tmp = {};
                            _.each(f.opaque.crew, function (person) {
                                if (!tmp[person.activity])
                                    tmp[person.activity] = [];
                                tmp[person.activity].push(person.name + " " + person.surname);
                            })
                            _.each(tmp, function(people, activity) {
                                f.extra_infos[activity] = tmp[activity].join(', ');
                            })
                        }

                        movies[f._id] = f;
                        movies[f._id].days = [];
                    }

                    /* Filter screening pricings */
                    var i = 0;
                    _.each(s.pricings, function(p, key) {
                        if (!_.contains(p.sellers, "eshop"))
                        delete s.pricings[key];
                    });

                    delete s.films;

                    if (!_.contains(movies[f._id].days, day))
                        movies[f._id].days.push(day);

                    movies[f._id].screenings.push(s);
                    movies[f._id].section = movies[f._id].opaque.section;
                })
            });

            days.sort();
            callback(null, movies, days);
        });
    };

    return {
        /* Cache mechanism*/
        loaded: false,
        /* Enhanced movies array */
        movies: [],
        /* Days array */
        days: [],

        /**
         * Get all the movies and screenings.
         * Doesn't call the API if we already did it.
         *
         * @param function: Callback which receives the movies and the screenings
         */
        all: function(callback) {
            if (this.loaded)
                return (callback && callback(null, this.movies, this.days));

            return this.reload(callback);
        },

        /**
         * Get a movie by his id or ref
         *
         * @param mixed:    The movie id or ref
         * @param boolean:  True to get the movie only from the cache.
         *                  If false, call the API only if not loaded.
         * @param function: Callback which receives the movie
         */
        get: function(movie_id, from_cache, callback) {

            /* Call the API only if needed */
            this.all(function(err, movies, days) {
                if (err)
                    return (callback && callback(err));

                _.each(movies, function(m) {

                    /* Search by id */
                    if (m._id == movie_id)
                        return (callback && callback(null, m));

                    /* Search by ref */
                    if (!_.isEmpty(m.refs)) {
                        _.each(m.refs, function(r) {
                            if (r.id == movie_id) {
                                return (callback && callback(null, m));
                            }
                        });
                    }
                });
            });

            /* If already loaded, directly return the movie */
            if (this.loaded) {
                var movie;

                _.each(this.movies, function(m) {
                    /* Search by id */
                    if (m._id == movie_id)
                        movie = m;

                    /* Search by ref */
                    if (!_.isEmpty(m.refs)) {
                        _.each(m.refs, function(r) {
                            if (r.id == movie_id) {
                                movie = m;
                            }
                        });
                    }
                });

                return movie;
            }
        },

        /**
         * Get all the screenings.
         * The screenings encapsulate their movie in a "movie" field.
         * Doesn't call the API if we already did it.
         *
         * @param mixed:    The movie id or ref
         * @param boolean:  True to get the movie only from the cache.
         *                  If false, call the API only if not loaded.
         * @param function: Callback which receives the screening
         */
        screenings: function(callback) {
            var screenings = [];

            /* Call the API only if needed */
            this.all(function(err, movies, days) {
                if (err)
                    return (callback && callback(err));

                /* Enhance with the movie */
                _.each(movies, function(m) {
                    _.each(m.screenings, function(s) {
                        s.movie = m;
                        screenings.push(s);
                    });
                });

                callback && callback(null, screenings);
            });
        },

        /**
         * Get a screening by his id or ref
         *
         * @param mixed:    The movie id. Optional.
         * @param mixed:    The screening id or ref
         * @param boolean:  True to get the movie only from the cache.
         *                  If false, call the API only if not loaded.
         * @param function: Callback which receives the movie
         */
        screening: function(movie_id, screening_id, callback) {
            /* Call the API only if needed */
            this.all(function(err, movies, days) {
                if (err)
                    return (callback && callback(err));

                _.each(movies, function(m) {
                    if (!_.isNull(movie_id) && m._id != movie_id)
                        return false;

                    _.each(m.screenings, function(s) {
                        /* Search on id */
                        if (s._id == screening_id) {
                            return (callback && callback(null, m, s));
                        }

                        /* Search on ref */
                        if (!_.isEmpty(s.refs)) {
                            _.each(s.refs, function(r) {
                                if (r.id == screening_id) {
                                    return (callback && callback(null, m, s));
                                }
                            });
                        }
                    });
                });
            });
        },

        /**
         * Get all the movies from the remote server.
         * Sets the loaded flag to true when finished.
         *
         * @param function: Callback which receives the movies and the days
         */
        reload: function(callback) {
            var self = this;
            /* Load from the remote server and enhance the movies */
            load_movies(function(err, movies, days) {
                if (err)
                    return (callback && callback(err));

                self.loaded = true;
                self.movies = movies;
                self.days   = days;

                return (callback && callback(null, self.movies, self.days));
            });
        },
    };
});
