/**
 * Tickets Service
 *
 * Uses the Tiketack API following calls:
 *      - login with ticket
 *          POST /ticket/view/:{ticket_number,:ticket:key}
 *      - check a screening bookability
 *          GET  /screening/bookability/:screening_id
 *      - book on ticket
 *          GET  /screening/book_on_ticket/:screening_id
 *      - cancel a booking
 *          POST /ticket/cancel_booking_json/:{booking_id}
 *
 * To bypass the Captcha mechanism, a token is sent by the server when we get
 * the cart. This token is sent in all following calls as header X-Cap-Token.
 * To maintain the session, the PHPSESSID is also sent by the server when we
 * get the cart.
 *
 * @author Belkacem Alidra <belkacem.alidra@netoxygen.ch>
 */
TicketackServices.factory(
    'Tickets',
    [ 'Cart', 'Movies', 'Config', 'LocalStorage', '$resource', '_',
    function(Cart, Movies, Config, LocalStorage, $resource, _) {

        /**
         * Ressource used to communicate with the engine.
         * It ensures to always send the configured api key.
         */
        var Ticket = $resource(Config.ticketack_url + '/api/tickets/:uuid', /* default params */null, {
            "get": {
                "method": "GET",
                "headers": {
                    "X-API-Key": Config.api_key
                },
                "isArray": false,
                "timeout": 6000
            }
        });

        /**
         * Add to the bought or scanned one time ticket informations about the movie and
         * screening.
         *
         * @param array: The activated items
         * @return array: The enhanced activated items
         */
        var enhance_items = function(items) {
            _.each(items, function(i) {
                if (i.type === "screening") {
                    Movies.screening(null, i.item_id, function(err, movie, screening) {
                        i.movie     = movie;
                        i.screening = screening;
                    });
                }
            });

            return items;
        };

        return {
            /**
             * Array of bought one time pass
             */
            "one_times": [],
            /**
             * Array of bought pass
             */
            "pass": [],

            /**
             * The current pass
             */
             "current_pass": null,

            /**
             * Fetch the activated items from the eshop payment screen.
             * Some magic stuff to transfer activated items from the eshop angular
             * scope to the PaymentCtrl scope.
             *
             * Items are like this:
             * - name:      Name of the cart item
             * - type:      Can be screening, pass or article
             * - ticket_id: The ticket id if any
             * - item_id:   The screening or ticket type or article id, dependeing on type
             * - pdf_url:   The PDF url, can be null if no PDF is available
             *
             * We get these items and store them in the Local Storage.
             */
            store_activated_items: function(activated_items, callback) {
                if (_.isEmpty(activated_items))
                    return (callback && callback(null, this.one_times, this.pass));

                var self = this;
                var pass = [];
                _.each(activated_items, function(i) {
                    if ("screening" === i.type)
                        self.one_times.push(i);
                    else if ("pass" === i.type)
                        pass.push(i);
                });

                /** To authenticate a pass, we need the number and key.
                 *  Let's call the engine for each pass.
                 */
                var fetch_multi_tickets = function(activated_pass, callback) {
                    var pass = activated_pass.pop();

                    if (undefined == pass)
                        return (callback && callback());

                    Ticket.get({"uuid": pass.ticket_id}, function(ticket) {
                        callback && callback(pass, ticket);

                        fetch_multi_tickets(activated_pass, callback);
                    });
                };

                fetch_multi_tickets(pass, function(activated_item, ticket_from_engine) {
                    if (!activated_item) {
                        /* No more item. We can save everything. */
                        LocalStorage.setArray('one_times', self.one_times);
                        LocalStorage.setArray('pass', self.pass);

                        return (callback && callback(
                            null,
                            enhance_items(self.one_times),
                            self.pass
                        ));
                    }

                    /* Pick some infos from the engine ticket */
                    activated_item.number = ticket_from_engine.vdr_auth_serial;
                    activated_item.key    = ticket_from_engine.vdr_auth_key;

                    self.pass.push(activated_item);
                    LocalStorage.setArray('pass', self.pass);

                    /* Let's activate just imported pass */
                    self.set_current_pass(activated_item, function(err, current_pass) {});
                });
            },

            /**
             * Get the one times and pass from the Local Storage
             * Return enhanced items.
             *
             * @param function: Callback function which receives the one times
             *                  and pass.
             */
            get: function(callback) {
                this.one_times = enhance_items(LocalStorage.getArray('one_times', []));
                this.pass      = LocalStorage.getArray('pass', []);

                callback && callback(null, this.one_times, this.pass);
            },

            /**
             * Get a pass by his ticket id
             *
             * @param integer|string: Ticket id
             * @param function: Callback function which receives the pass
             */
            get_pass: function(ticket_id, callback) {
                this.get(function(err, one_times, pass) {
                    if (err)
                        return (callback && callback(err));

                    _.each(pass, function(p) {
                        if (p.ticket_id == ticket_id)
                            return (callback && callback(null, p));
                    });
                });
            },

            /**
             * Get the current logged in pass
             */
            get_current_pass: function() {
                return this.current_pass;
            },

            /**
             * Set the current pass
             *
             * @param integer|string: The id of the pass to set as current
             * @param function: Callback which receives the current pass
             */
            set_current_pass: function(ticket_id, callback) {
                var self = this;
                this.get_pass(ticket_id, function(err, pass) {
                    if (err)
                        return (callback && callback(err));

                    self.login(pass,
                        function(err, pass) {
                            if (err)
                                return (callback && callback(err));

                            return (callback && callback(null, self.current_pass));
                        }
                    );
                });
            },

            /**
             * Ticket authentication with number and key
             *
             * Authenticates the user ticket
             * On success, we store the ticket in the current_pass.
             * The ticket bookings are enhanced with the movies
             * and screenigs infos.
             *
             * @param number: The ticket number
             * @param key: The ticket key
             * @param callback
             */
            login_with_ticket_number: function(number, key, callback) {
                var self = this;
                return Cart.http_post(
                    '/ticket/view_json/',
                    { "ticket_number": number, "ticket_key": key },
                    function(err, infos) {
                        /* infos.data is like this:
                        {
                            - id: ""
                            - contact: {}
                            - bookings: []
                        }
                        */
                        if (err) {
                            self.current_pass    = undefined;
                            return (callback && callback(err));
                        }

                        var pass       = infos.data;
                        pass.number    = number;
                        pass.key       = key;
                        pass.ticket_id = pass.id;

                        /* enhance ticket bookings */
                        if (!_.isEmpty(pass.bookings)) {
                            _.each(pass.bookings, function(booking) {
                                Movies.screening(null, booking.screening_id, function(err, movie, screening) {
                                    if (err)
                                        return false;
                                    booking.movie     = movie;
                                    booking.screening = screening;
                                });
                            });
                        }

                        self.current_pass = pass;

                        return (callback && callback(null, self.current_pass));
                    }
                );
            },

            /**
             * Ticket authentication shortcut
             *
             * @param object: The pass to login
             * @param callback
             */
            login: function(pass, callback) {
                return this.login_with_ticket_number(pass.number, pass.key, callback);
            },

            /**
             * Get a ticket by his scanned URL
             * We fetch the ticket data from the engine, and
             * create an item stored in one_times or pass.
             *
             * @param string: The ticket URL
             */
            fetch_ticket_by_url: function(url, callback) {
                var self = this;

                /* Ask the eshop for the ticket id */
                return Cart.http_post(
                    '/ticket/view_json/',
                    { "url": url },
                    function(err, infos) {
                        if (err) {
                            return (callback && callback(err));
                        }

                        var uuid = infos.data.id;

                        /* Nox get the ticket data from the engine */
                        Ticket.get({"uuid": uuid}, function(ticket) {

                            var type = ticket.type._id == "one-time-pass" ? "screening" : "pass";

                            /* Check if not already bought or imported */
                            var already_exists = false;
                            var existing       = (type == "pass" ? self.pass : self.one_times);
                            if (!_.isEmpty(existing)) {
                                _.each(existing, function(p) {
                                    if (p.ticket_id == ticket._id)
                                        already_exists = true;
                                });
                            }
                            if (already_exists)
                                return (callback && callback({"error": "Ce billet existe déjà"}));

                            var item = {
                                "ticket_id": ticket._id,
                                "type": type
                            };

                            if (type == "screening") {
                                item.item_id = ticket.type.screening_id;

                                /* Get the movie and screening infos if any */
                                item = enhance_items([item])[0];
                                self.one_times.push(item);
                                LocalStorage.setArray('one_times', self.one_times);

                                return (callback && callback(null, ticket));
                            }
                            else {
                                item.name = (
                                    "Abonnement" + " " +
                                    ticket.contact.firstname + " " + ticket.contact.lastname + " " +
                                    ticket.activated_pricing.name.fr
                                );
                                item.number = ticket.vdr_auth_serial;
                                item.key    = ticket.vdr_auth_key;
                                self.pass.push(item);
                                LocalStorage.setArray('pass', self.pass);

                                /* Let's activate just imported pass */
                                self.set_current_pass(item.ticket_id, function(err, current_pass) {
                                    return (callback && callback(null, ticket));
                                });
                            }
                        });
                    }
                );
            },

            /**
             * Screening bookability check
             *
             * Check if the user can book the screening with the current ticket.
             * The result is an object with the following boolean keys:
             *   - ticket_logged_in
             *   - screening_already_booked
             *   - ticket_can_book_screening
             *   - screening_seats
             *   - screening_exists
             *
             * @param screening_id: The screening ID
             * @param callback
             */
            check_bookability: function(screening_id, callback) {

                var url = '/screening/bookability?id=' + screening_id +
                          '&PHPSESSID=' + Cart.session_id +
                          '&pos_client_side_request=true';

                return Cart.http_get(url, callback);
            },

            /**
             * Book a screening on the current ticket
             *
             * @param screening_id: The screening ID
             * @param callback
             */
            book_on_ticket: function(screening_id, callback) {

                var url = '/screening/book_on_ticket?id=' + screening_id +
                          '&PHPSESSID=' + Cart.session_id +
                          '&pos_client_side_request=true';

                return Cart.http_get(url, callback);
            },

            /**
             * Cancel a book screening on the current ticket
             *
             * @param booking_id: The booking ID
             * @param callback
             */
            cancel_booking: function(booking_id, callback) {

                return Cart.http_post(
                    '/ticket/cancel_booking_json',
                    { "id": booking_id, "pos_client_side_request": "json" },
                    callback
                );
            }
        }
    }
]);
